#pragma once

#ifdef USB_UNIQUEKEY_DLL_EXPORT
#define USB_UNIQUEKEY_DLL_IMPORT_EXPORT __declspec(dllexport)
#else
#define USB_UNIQUEKEY_DLL_IMPORT_EXPORT __declspec(dllimport)
#endif

#define UNIQUE_USB_KEY_TYPE_STRONG 0x01
#define UNIQUE_USB_KEY_TYPE_VENDOR 0x02
#define UNIQUE_USB_KEY_TYPE_VENDOR_AND_PRODUCT 0x03

#define INVALID_PORT_NUMBER (0xFFFFFFFF)
#define INVALID_DISK_SIZE (0xFFFFFFFFFFFFFFFF)

#define MAX_UNIQUE_KEY_LENGTH (32)
#define MAX_USB_HARDWARE_SERIALNUMBER_LENGTH (50)

//
// korean:
// VendorId: ġ  ̵Դϴ.
// ProductId: ġ ǰ ̵Դϴ.
// ProductRevisionLevel: ǰ  ȣԴϴ.
//
// English:
//
#pragma pack(1)
typedef struct _PHYSICAL_DEVICE_INQUIRY_ID
{
	CHAR VendorId[50];
	CHAR ProductId[50];
	CHAR ProductRevisionLevel[10];
} PHYSICAL_DEVICE_INQUIRY_ID, *PPHYSICAL_DEVICE_INQUIRY_ID;
#pragma pack()

#ifdef __cplusplus
extern "C"
{
#endif
//
// korean:
// Function Name: GetUSBHardwareUniqueKey
//  USB ġ ϵ Ű ɴϴ.
//
// Parameters:
//
// UniqueUSBKeyType: Ű Ÿ ϸ Ű Ÿ Ʒ ϴ.
// 
// UNIQUE_USB_KEY_TYPE_STRONG -
//  ,   ũ ũ⸦  USB ġ ̸ Ͽ, ϵ  Ű ɴϴ.
// Ư USBġ  Ű   Ÿ ϼ.
//
// UNIQUE_USB_KEY_TYPE_VENDOR - 
//  (Vendor)̸  USBġ鿡 ؼ  ϵ Ű ɴϴ.
// Ư  ϴ USBġ Ϸ  Ÿ ϼ.
//
// UNIQUE_USB_KEY_TYPE_VENDOR_AND_PRODUCT - 
//  (Vendor) ǰ(Product)̸  USBġ鿡 ؼ  ϵ Ű ɴϴ.
// Ư  ϸ, ǰ̸  USBġ Ϸ  Ÿ ϼ.
//
// ߿(Important): UniqueUSBKeyType ŸԿ   USBġ  Ű ٸϴ. ̰ Ű ο  ŸԿ  ĺ ԵǾ ֱ Դϴ.
//
// DetectPort:   TRUE ,  USBġ (Attached)  Ʈȣ νϿ Ʈȣ  ٸ ϵ Ű ɴϴ.
//
// DrivePath:  USBġ Դϴ.
//
// UniqueKey: ϵǴ Ű̸ 32Ʈ ƼƮ (Multi-Byte)ڿԴϴ.
//
// Return Value:
//
//   TRUE մϴ.
//
//
// English:
USB_UNIQUEKEY_DLL_IMPORT_EXPORT BOOL __cdecl GetUSBHardwareUniqueKey(
	__in ULONG UniqueUSBKeyType,
	__in BOOL DetectPort,
	__in WCHAR *DrivePath,
	__out CHAR *UniqueKey);

//
// korean:
// Function Name: GetDeviceIndex
// ο شϴ ġ ε մϴ.
//
// Parameters:
//
// DrivePath:  USBġ Դϴ.
//
// Return Value:
//
// ũ ε մϴ.
//
// English:
USB_UNIQUEKEY_DLL_IMPORT_EXPORT ULONG GetDeviceIndex(const WCHAR *DrivePath);

//
// korean:
// Function Name: GetPhysicalDeviceID
// ο شϴ  ̽  оɴϴ.
//
// Parameters:
//
// DrivePath:  USBġ Դϴ.
//
// pPhysicalInquiryId:    ü̸, PHYSICAL_DEVICE_INQUIRY_ID Դϴ.
//
// Return Value:
//
//   TRUE մϴ.
//
// English:
USB_UNIQUEKEY_DLL_IMPORT_EXPORT BOOL GetPhysicalDeviceID(WCHAR *DrivePath, PPHYSICAL_DEVICE_INQUIRY_ID pPhysicalInquiryId);

//
// korean:
// Function Name: IsBusTypeUSB
// ο شϴ ġ USB Ÿ üũϸ, ߰ ̵ ̵ üũմϴ.
//
// Parameters:
//
// DrivePath:  ġ Դϴ.
//
// pRemovableMedia:   NULL ƴ  ̵ ̵  ΰ pRemovableMedia  ˴ϴ.
//
// Return Value:
//
//   TRUE մϴ.
//
// English:
USB_UNIQUEKEY_DLL_IMPORT_EXPORT BOOL IsBusTypeUSB(WCHAR *DrivePath, PBOOL pRemovableMedia);

//
// korean:
// Function Name: GetDiskSize
// ο شϴ ġ ũ ũ⸦ оɴϴ.
//
// Parameters:
//
// DrivePath:  ġ Դϴ.
//
// Return Value:
//
//   ũ ũ⸦ ϸ,   INVALID_DISK_SIZE մϴ.
//
// English:
USB_UNIQUEKEY_DLL_IMPORT_EXPORT ULONGLONG GetDiskSize(const WCHAR *DrivePath);

//
// korean:
// Function Name: GetUSBPortNumber
// ġ εȣκ  USBƮȣ оɴϴ.
//
// Parameters:
//
// DeviceIndex: ġ ε ȣԴϴ.
//
// Return Value:
//
//    USBƮȣ ϸ,   INVALID_PORT_NUMBER մϴ.
//
// English:
USB_UNIQUEKEY_DLL_IMPORT_EXPORT ULONG GetUSBPortNumber(ULONG DeviceIndex);

//
// korean:
// Function Name: GetUSBHardwareSerialNumber
// ġ εȣκ  USB øȣ оɴϴ.
//
// Parameters:
//
// DeviceIndex: ġ ε ȣԴϴ.
//
// SerialNumber: øȣ ް  ڵ ڿ Դϴ.
//
// uSerialNumberLength: øȣ ް  ڵ ڿ Դϴ. ( : ڿ BytesũԴϴ. )
//
// Return Value:
//
//   TRUE մϴ.
//
// English:
USB_UNIQUEKEY_DLL_IMPORT_EXPORT BOOL GetUSBHardwareSerialNumber(
	ULONG DeviceIndex,
	WCHAR *SerialNumber,
	ULONG uSerialNumberLength);

#ifdef __cplusplus
}
#endif

#ifndef USB_UNIQUEKEY_DLL_EXPORT
#ifdef _WIN64
#pragma comment(lib, "Lib\\USBUniquekey64.lib")
#else
#pragma comment(lib, "Lib\\USBUniquekey.lib")
#endif
#endif